@file:JvmName("_AutocloseKt")

package ru.landgrafhomyak.db.serdha0.user_commons.executors

import kotlin.contracts.InvocationKind
import kotlin.contracts.contract
import kotlin.jvm.JvmName


@Suppress("FunctionName")
@PublishedApi
internal inline fun <R> _safeAutoClose(onAbort: () -> Unit = {}, action: () -> R): R {
	contract {
		callsInPlace(action, InvocationKind.EXACTLY_ONCE)
		callsInPlace(onAbort, InvocationKind.AT_MOST_ONCE)
	}
	val ret: R
	try {
		ret = action()
	} catch (e1: Throwable) {
		try {
			onAbort()
		} catch (e2: Throwable) {
			e1.addSuppressed(e2)
		}
		throw e1
	}
	return ret
}