@file:OptIn(LowLevelApi::class)
@file:JvmName("_Row2Table")
@file:Suppress("DuplicatedCode")

package ru.landgrafhomyak.db.serdha0.user_commons.executors

import kotlin.jvm.JvmName
import ru.landgrafhomyak.db.serdha0.api.LowLevelApi
import ru.landgrafhomyak.db.serdha0.api.queries._Query
import ru.landgrafhomyak.db.serdha0.api.runtime.Transaction

public suspend inline fun <qUE : Any> Transaction.select(
	compiledQuery: _Query.Params2Table<qUE>,
	params: RowProducer0<qUE>,
	transform: RowConsumer0<qUE, Unit>
) {
	val iRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { iRow._abort() }) {
		params.initializeRow(iRow)
	}
	val oRow = iRow._finish()
	_safeAutoClose(onAbort = { oRow._abort() }) {
		while (oRow._next())
			transform.transformRow(oRow)
	}
	oRow._finish()
}

public suspend inline fun <qUE : Any> Transaction.select(
	compiledQuery: _Query.Void2Table<qUE>,
	transform: RowConsumer0<qUE, Unit>
) {
	val oRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { oRow._abort() }) {
		while (oRow._next())
			transform.transformRow(oRow)
	}
	oRow._finish()
}

public suspend inline fun <qUE : Any, E> Transaction.mapRows(
	compiledQuery: _Query.Params2Table<qUE>,
	params: RowProducer0<qUE>,
	transform: RowConsumer0<qUE, E>
): List<E> {
	val iRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { iRow._abort() }) {
		params.initializeRow(iRow)
	}
	val oRow = iRow._finish()
	val out = ArrayList<E>()
	_safeAutoClose(onAbort = { oRow._abort() }) {
		while (oRow._next())
			out.add(transform.transformRow(oRow))
	}
	oRow._finish()
	return out
}

public suspend inline fun <qUE : Any, E> Transaction.mapRows(
	compiledQuery: _Query.Void2Table<qUE>,
	transform: RowConsumer0<qUE, E>
): List<E> {
	val oRow = this._executeQuery(compiledQuery)
	val out = ArrayList<E>()
	_safeAutoClose(onAbort = { oRow._abort() }) {
		while (oRow._next())
			out.add(transform.transformRow(oRow))
	}
	oRow._finish()
	return out
}

public suspend inline fun <qUE : Any, E> Transaction.mapRowsIndexed(
	compiledQuery: _Query.Params2Table<qUE>,
	params: RowProducer0<qUE>,
	firstIndex: Int = 0,
	transform: RowConsumer1<qUE, E>
): List<E> {
	val iRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { iRow._abort() }) {
		params.initializeRow(iRow)
	}
	val oRow = iRow._finish()
	val out = ArrayList<E>()
	_safeAutoClose(onAbort = { oRow._abort() }) {
		var i = firstIndex
		while (oRow._next())
			out.add(transform.transformRow(oRow, i++))
	}
	oRow._finish()
	return out
}

public suspend inline fun <qUE : Any, E> Transaction.mapRowsIndexed(
	compiledQuery: _Query.Void2Table<qUE>,
	firstIndex: Int = 0,
	transform: RowConsumer1<qUE, E>
): List<E> {
	val oRow = this._executeQuery(compiledQuery)
	val out = ArrayList<E>()
	_safeAutoClose(onAbort = { oRow._abort() }) {
		var i = firstIndex
		while (oRow._next())
			out.add(transform.transformRow(oRow, i++))
	}
	oRow._finish()
	return out
}

public suspend inline fun <qUE : Any, E, C : MutableCollection<E>> Transaction.mapRowsTo(
	compiledQuery: _Query.Params2Table<qUE>,
	params: RowProducer0<qUE>,
	dst: C,
	transform: RowConsumer0<qUE, E>
): C {
	val iRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { iRow._abort() }) {
		params.initializeRow(iRow)
	}
	val oRow = iRow._finish()
	_safeAutoClose(onAbort = { oRow._abort() }) {
		while (oRow._next())
			dst.add(transform.transformRow(oRow))
	}
	oRow._finish()
	return dst
}

public suspend inline fun <qUE : Any, E, C : MutableCollection<E>> Transaction.mapRowsTo(
	compiledQuery: _Query.Void2Table<qUE>,
	dst: C,
	transform: RowConsumer0<qUE, E>
): C {
	val oRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { oRow._abort() }) {
		while (oRow._next())
			dst.add(transform.transformRow(oRow))
	}
	oRow._finish()
	return dst
}

public suspend inline fun <qUE : Any, E, C : MutableCollection<E>> Transaction.mapRowsIndexedTo(
	compiledQuery: _Query.Params2Table<qUE>,
	params: RowProducer0<qUE>,
	dst: C, firstIndex: Int = 0,
	transform: RowConsumer1<qUE, E>
): C {
	val iRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { iRow._abort() }) {
		params.initializeRow(iRow)
	}
	val oRow = iRow._finish()
	_safeAutoClose(onAbort = { oRow._abort() }) {
		var i = firstIndex
		while (oRow._next())
			dst.add(transform.transformRow(oRow, i++))
	}
	oRow._finish()
	return dst
}

public suspend inline fun <qUE : Any, E, C : MutableCollection<E>> Transaction.mapRowsIndexedTo(
	compiledQuery: _Query.Void2Table<qUE>,
	dst: C, firstIndex: Int = 0,
	transform: RowConsumer1<qUE, E>
): C {
	val oRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { oRow._abort() }) {
		var i = firstIndex
		while (oRow._next())
			dst.add(transform.transformRow(oRow, i++))
	}
	oRow._finish()
	return dst
}

public suspend inline fun <qUE : Any, E> Transaction.mapRowsTo(
	compiledQuery: _Query.Params2Table<qUE>,
	params: RowProducer0<qUE>,
	dst: Array<E>, dstOffset: Int = 0,
	transform: RowConsumer0<qUE, E>
): Array<E> {
	val iRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { iRow._abort() }) {
		params.initializeRow(iRow)
	}
	val oRow = iRow._finish()
	_safeAutoClose(onAbort = { oRow._abort() }) {
		var i = dstOffset
		while (oRow._next())
			dst[i++] = transform.transformRow(oRow)
	}
	oRow._finish()
	return dst
}

public suspend inline fun <qUE : Any, E> Transaction.mapRowsTo(
	compiledQuery: _Query.Void2Table<qUE>,
	dst: Array<E>, dstOffset: Int = 0,
	transform: RowConsumer0<qUE, E>
): Array<E> {
	val oRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { oRow._abort() }) {
		var i = dstOffset
		while (oRow._next())
			dst[i++] = transform.transformRow(oRow)
	}
	oRow._finish()
	return dst
}